/*
 * Copyright (C) 2004 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * $Log$
 */

#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "propertyview.h"

wxPropertyValidatorRegistry listValReg;

BEGIN_EVENT_TABLE(CPropertyView, wxPropertyListView)
    EVT_BUTTON(wxID_OK, CPropertyView::OnOK)
END_EVENT_TABLE()

/*!
 * \brief Constructor.
 */
CPropertyView::CPropertyView(CNutNode *nut, CUdpThread *thread)
    : wxPropertyListView(NULL, wxPROP_BUTTON_OK | wxPROP_BUTTON_CANCEL | wxPROP_BUTTON_CHECK_CROSS | wxPROP_SHOWVALUES)
    , m_nut(nut)
    , m_thread(thread)
{
    listValReg.RegisterValidator((wxString)_T("real"), new wxRealListValidator);
    listValReg.RegisterValidator((wxString)_T("string"), new wxStringListValidator);
    listValReg.RegisterValidator((wxString)_T("integer"), new wxIntegerListValidator);
    listValReg.RegisterValidator((wxString)_T("bool"), new wxBoolListValidator);
    listValReg.RegisterValidator((wxString)_T("stringlist"), new wxListOfStringsListValidator);
    AddRegistry(&listValReg);

    m_sheet = new wxPropertySheet;
    m_sheet->AddProperty(new wxProperty(_T("MAC"), m_nut->m_mac, _T("string")));
    m_sheet->AddProperty(new wxProperty(_T("IP"), m_nut->m_cip_addr, _T("string")));
    m_sheet->AddProperty(new wxProperty(_T("Mask"), m_nut->m_ip_mask, _T("string")));
    m_sheet->AddProperty(new wxProperty(_T("Gate"), m_nut->m_gateway, _T("string")));
    m_sheet->AddProperty(new wxProperty(_T("Host"), m_nut->m_hostname, _T("string")));
}

void CPropertyView::ShowView(wxPanel *panel)
{
    wxPropertyListView::ShowView(m_sheet, panel);
}

void CPropertyView::OnOK(wxCommandEvent& event)
{
    /*
     * Retrieve currently edited item, if any.
     */
    OnCheck(event);

    if(wxMessageBox("Update?", "Setup", wxYES_NO) == wxYES) {

        DISCOVER *tele = (DISCOVER *)malloc(sizeof(DISCOVER));
        memcpy(tele, m_nut->m_tele, sizeof(DISCOVER));

        tele->tele_type = 2;

        char *str = m_sheet->GetProperty("Host")->GetValue().StringValue();
        memset(tele->tele_hostname, 0, sizeof(tele->tele_hostname));
        strncpy((char *)tele->tele_hostname, str, sizeof(tele->tele_hostname) - 1);

        str = m_sheet->GetProperty("MAC")->GetValue().StringValue();
        CNutNode::StringToMac(str, tele->tele_mac);

        str = m_sheet->GetProperty("IP")->GetValue().StringValue();
        tele->tele_cip_addr = CNutNode::StringToIp(str);

        str = m_sheet->GetProperty("Mask")->GetValue().StringValue();
        tele->tele_ip_mask = CNutNode::StringToIp(str);

        str = m_sheet->GetProperty("Gate")->GetValue().StringValue();
        tele->tele_gateway = CNutNode::StringToIp(str);

        m_thread->Broadcast(tele);
    }
    wxPropertyListView::OnOk(event);
}

bool CPropertyView::OnClose()
{
    m_thread->Broadcast();

    wxPropertyListView::OnClose();

    return true;
}
