/*
 * Copyright (C) 2004 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * $Log$
 */

#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

#include <wx/thread.h>

#if !wxUSE_THREADS
#error "This sample requires thread support!"
#endif

#include "udpthread.h"
#include "discover.h"
#include "nutnode.h"
#include "nutdisc.h"

/*!
 * \brief Constructor.
 */
CUdpThread::CUdpThread(CMainFrame *frame)
    : wxThread()
    , m_frame(frame)
{
}

void CUdpThread::OnExit()
{
}

bool CUdpThread::Broadcast(DISCOVER *disc)
{
    int sock;
    int on = 1;
    struct sockaddr_in client;

    if((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
         return false;
    }
    setsockopt(sock, SOL_SOCKET, SO_BROADCAST, (char *)&on, sizeof(on));

    client.sin_family = AF_INET;
    client.sin_port = htons(DISCOVER_PORT);
    client.sin_addr.s_addr = INADDR_BROADCAST;
    if(disc) {
        sendto(sock, (char *)disc, sizeof(DISCOVER), 0, (struct sockaddr *)&client, sizeof(client));
    }
    else {
        DISCOVER dummy;
        memset(&dummy, 0, sizeof(dummy));
        sendto(sock, (char *)&dummy, sizeof(DISCOVER), 0, (struct sockaddr *)&client, sizeof(client));
    }
    closesocket(sock);

    return true;
}

/*
 * Start the thread.
 */
void *CUdpThread::Entry()
{
    int sock;
    unsigned short lport = DISCOVER_PORT;
    struct sockaddr_in server;
    struct sockaddr_in client;
    int cl;
    int on = 1;
    char buf[1500];
    DISCOVER *tele;

    if((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
         return NULL;
    }
    setsockopt(sock, SOL_SOCKET, SO_BROADCAST, (char *)&on, sizeof(on));
    server.sin_family = AF_INET;
    server.sin_port = htons(lport);
    server.sin_addr.s_addr = INADDR_ANY;
    if(bind(sock, (struct sockaddr *)&server, sizeof(server)) < 0) {
         return NULL;
    }

    Broadcast();
    for(;;) {
        cl = sizeof(client);
        if(recvfrom(sock, buf, sizeof(buf), 0, (struct sockaddr *)&client, &cl) >= sizeof(DISCOVER)) {
            tele = (DISCOVER *)buf;
            if(tele->tele_type == 1) {
                tele = (DISCOVER *)malloc(sizeof(DISCOVER));
                memcpy(tele, buf, sizeof(DISCOVER));
                wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, UDP_EVENT);
                event.SetInt((int)tele);
                wxPostEvent(m_frame, event);
            }
        }
    }
    return NULL;
}

